const {
	      byteArray,
	      gi  : { Gio: { SettingsSchemaSource, Settings }, GLib },
	      ui  : {
		      main     : { panel: { statusArea: { aggregateMenu: { _powerProfiles: { _item: { menu } } } } } },
		      popupMenu: { PopupSeparatorMenuItem, PopupSwitchMenuItem }
	      },
	      misc: { extensionUtils },
	      gettext
      }             = imports;
const { uuid, dir } = extensionUtils.getCurrentExtension();

let mySeparator;
let mySwitch;
let myMenuConnection;
let mySwitchConnection;
// 0: intel_pstate
// 1: acpi-cpufreq
// 2: CPU not supported
let myCpuType = 2;

function init()
{
	gettext.bindtextdomain( uuid, dir.get_child( 'locale' ).get_path() );
	gettext.textdomain( uuid );

	const state    = getBoostState();
	const settings = getSettings();

	// Turn off if it is on now, user has opted-in for persistence and the last time he turned off Boost
	if ( state && settings.get_boolean( 'persist' ) && !settings.get_boolean( 'boost' ) )
	{
		GLib.timeout_add( 0, 5000, _ => setBoostState( false ) );
	}
	else
	{
		// Always set state. This clears use cases of this sort:
		// 1. Disable Boost
		// 2. Reboot (this will enable Boost)
		// 3. Enable Persist (while Boost is on)
		// 4. Reboot
		// User will be prompted to disable Boost since setting is remembered from the last time he clicked the switch.
		// Clear it here, so that doesn't happen.
		settings.set_boolean( 'boost', state );
	}
}

function enable()
{
	myMenuConnection = menu.connect( 'open-state-changed', ( { box }, open ) =>
	{
		if ( open )
		{
			const state = getBoostState();

			// Do not add switch if CPU not supported
			if ( state !== null )
			{
				mySeparator = new PopupSeparatorMenuItem;
				mySwitch    = new PopupSwitchMenuItem( gettext.gettext( 'Frequency Boost' ), state );

				mySwitchConnection = mySwitch.connect( 'toggled', ( { _switch: { state } } ) => setBoostState( state ) );

				box.insert_child_at_index( mySeparator, 0 );
				box.insert_child_at_index( mySwitch, 0 );
			}
		}
		else
		{
			destroyMyItems();
		}
	} );
}

function disable()
{
	// Redundancy. Items are destroyed when menu closes, but still...
	destroyMyItems();

	menu.disconnect( myMenuConnection );
	myMenuConnection = null;

	// Removed since GNOME calls disable() when user locks the PC...
	// Bring back Boost to on (and save boolean)
	/*if ( !getBoostState() )
	 {
	 setBoostState( true );
	 }*/
}

function getSettings()
{
	return new Settings( {
		settings_schema: SettingsSchemaSource
			.new_from_directory( dir.get_child( 'schemas' ).get_path(), SettingsSchemaSource.get_default(), false )
			.lookup( 'org.gnome.shell.extensions.freq-boost-switch', true )
	} );
}

function getBoostState()
{
	return [
		// 0 means Boost is on, 1 means it is off
		[ 'intel_pstate/no_turbo', '0' ],
		// 0 means Boost is off, 1 means it is on
		[ 'cpufreq/boost', '1' ]
	].reduce( ( result, [ setting, enabledValue ], index ) =>
	{
		if ( result === null )
		{
			const out = GLib.spawn_command_line_sync( `bash -c "cat /sys/devices/system/cpu/${setting}"` )[ 1 ];

			if ( out.length )
			{
				myCpuType = index;

				return byteArray.toString( out ).trim() === enabledValue;
			}
		}

		return result;
	}, null );
}

function setBoostState( state )
{
	// Moved to the set_boost file. Reason - more meaningful pkexec message
	/*const [ setting, value ] = [
	 [ 'intel_pstate/no_turbo', Number( !state ) ],
	 [ 'cpufreq/boost', Number( state ) ]
	 ][ myCpuType ];

	 GLib.spawn_command_line_async( `pkexec bash -c "echo ${value} > /sys/devices/system/cpu/${setting}"` );*/

	GLib.spawn_command_line_async( `pkexec ${dir.get_child( 'set_boost' ).get_path()} "${myCpuType}" "${Number( state )}"` );

	// Unfortunately this cannot be moved to the set_boost file as it runs with root and gsettings is not happy.
	// This means we can get out of sync if user presses Cancel :(
	getSettings().set_boolean( 'boost', state );

	// Destroy init timeout
	return false;
}

function destroyMyItems()
{
	mySwitch?.disconnect( mySwitchConnection );

	mySeparator?.destroy();
	mySwitch?.destroy();

	mySeparator = mySwitch = mySwitchConnection = null;
}